/**********************************************************************
 * queue.h                                                   March 1999
 * Horms                                             horms@vergenet.net
 *
 * Data independent queue 
 *
 * Released under the terms of the GNU LGPL
 *
 **********************************************************************/
#ifndef QUEUEBERT
#define QUEUEBERT

#include <stdlib.h>
#include <stdio.h>

typedef void * perdition_queue_t;

struct struct_queue_member{
  struct struct_queue_member *next;
  struct struct_queue_member *prev;
  perdition_queue_t value;
};

typedef struct struct_queue_member queue_member;

typedef struct{
  queue_member *first;
  queue_member *last;
  int size;
}queue;


/**********************************************************************
 * create_queue
 * Create a new, empty queue
 * pre: q: pounter to an unintialised queue
 * return: new, empty queue
 *         NULL on error
 **********************************************************************/

queue *create_queue(void);


/**********************************************************************
 * push
 * push an element onto the queue
 * pre: q: queue
 *      value: elemnt to push onto the queue
 *      destroy_perdition_queue_t: pointer to a function to 
 *      destroy queue elements
 * post: queue with element added
 *       NULL on error. On error, where possible the queue is destroyed.
 **********************************************************************/

queue *push(
  queue *q, 
  perdition_queue_t value, 
  void (*destroy_perdition_queue_t)(const void *)
);


/**********************************************************************
 * pop 
 * Pop an element off the queue
 * pre: q: queue to pop the element off
 *      value: element removed from the queue is assigned to *value
 * post: queue with element removed
 *       NULL on error
 * Note: popping an emty queue results in NULL being returned
 **********************************************************************/

queue *pop(queue *q, perdition_queue_t *value);


/**********************************************************************
 * destroy_queue
 * Destroy a queue, destroying each element present in the
 * queue first
 * pre: q: queue to destroy
 *      destroy_perdition_queue_t: pointer to a function to destroy a queue
 *      element
 * post: queue and all elements in the queue are destroyed
 *       No retrurn value
 **********************************************************************/

void destroy_queue(queue *q, void (*destroy_perdition_queue_t)(const void *));


/**********************************************************************
 * queue_length
 * Return the number of elements in the queue
 * pre: q: queue to find the number of elements in
 * post: number of elements in the queue
 *       -1 on error
 **********************************************************************/

int queue_length(queue *q);
 

#endif
