/**********************************************************************
 * tcp_socket_port.c                                      December 1999
 * Horms                                             horms@vergenet.net
 *
 * Operations on port numbers
 *
 * Released under the GNU LGPL
 *
 **********************************************************************/

#include "tcp_socket.h"


/**********************************************************************
 * tcp_socket_port_portno
 * port number of a service given as a string either
 * the port number or the service name as per /etc/services
 * pre: port name as per /etc/services or port number as a string
 * return: port number
 *         0 on error or if port name cannot be found in /etc/services
 **********************************************************************/

unsigned short int tcp_socket_port_portno(const char *port){
  struct servent *ent;
  unsigned short int portno;

  if(tcp_socket_port_str_is_number(port)){
    portno=htons(atoi(port));
  }
  else{
    if((ent=getservbyname(port, "tcp"))==NULL){
      return(0);
    }
    portno=(unsigned short int)ent->s_port;
  }
  return(portno);
}


/**********************************************************************
 * tcp_socket_port_str_is_number
 * Test if a null terminated string is composed entirely of digits (0-9)
 * pre: String
 * return: 1 if string contains only digits and null teminator
 *         0 otherwise
 **********************************************************************/

int tcp_socket_port_str_is_number(const char *str){
  int offset;

  for(offset=strlen(str)-1;offset>-1;offset--){
    if(!isdigit(*(str+offset))){
      break;
    }
  }

  return(offset>-1?0:1);
}
