/**********************************************************************
 * options.c                                                   May 1999
 * Horms                                             horms@vergenet.net
 *
 * Parse command line arguments
 * Code based on man getopt(3)
 *
 * Released under the terms of the GNU GPL
 *
 **********************************************************************/

#include "options.h"

/**********************************************************************
 * makegdbm_options
 * parse command line arguments
 **********************************************************************/

makegdbm_options_t makegdbm_options(int argc, char **argv){
  int c;
  makegdbm_options_t options;

  /*Defaults*/
  options.undo=0;

  while (1){
#ifdef HAVE_GETOPT_LONG
    int option_index = 0;
    static struct option long_options[] = {
      {"help", 0, 0, 0},
      {"undo", 0, 0, 0},
      {"jain", 2, 0, 0},
      {"jane", 2, 0, 0},
      {"jayne", 2, 0, 0},
      {0, 0, 0, 0}
    };

    c = getopt_long (argc, argv, "hu", long_options, &option_index);
#else
    c = getopt (argc, argv, "hu");
#endif
    if (c == -1){
      break;
    }

    switch (c){
#ifdef HAVE_GETOPT_LONG
      case 0:
        if(strcmp(long_options[option_index].name,"help")==0){
          usage(0);
        }
        if(strcmp(long_options[option_index].name,"undo")==0){
          options.undo=1;
        }
        if(str_is_jain(long_options[option_index].name)){
          printf(
            "%s, Oath\n", 
            long_options[option_index].name
          );
        }
        break;
#endif
      case 'h':
        usage(0);
        break;
      case 'u':
        options.undo=1;
        break;
      case '?':
        usage(-1);
        break;
      default:
        printf ("?? getopt returned character code 0%o ??\n", c);
      }
  }

  if (optind+1<argc || argc==1) {
    usage(-1);
  }

  options.mapname=argv[argc-1];

  return(options);
}


/**********************************************************************
 * usage
 * Display usage information and exit
 * Prints to stdout if exit_status=0, stderr otherwise
 **********************************************************************/

void usage(int exit_status){
  FILE *stream;

  if(exit_status!=0){
     stream=stderr;
  }
  else{
     stream=stdout;
  }

    
  fprintf(
    stream, 
    "Usage: makegdbm [options] gdbmname\n"
#ifdef HAVE_GETOPT_LONG
    "   options: -h, --help: print this message\n"
    "            -u, --undo: print content of database file, one entry a line\n"
#else
    "   options: -h: print this message\n"
    "            -u: print content of database file, one entry a line\n"
#endif
  );

  exit(exit_status);
}
